@@ -37,11 +37,14 @@
 # storing result to a SciDB array when eval=TRUE.
 # name: (character) optional SciDB array name to store to
 # gc: (logical) optional, when TRUE tie result to R garbage collector
-scidbeval = function(expr, eval=TRUE, name, gc=TRUE)
+# temp (logical, optional): when TRUE store as a SciDB temp array
+scidbeval = function(expr, eval=TRUE, name, gc=TRUE, temp=FALSE)
 {
   ans = eval(expr)
   if(!(inherits(ans,"scidb") || inherits(ans,"scidbdf"))) return(ans)
-  .scidbeval(ans@name, `eval`=eval, name=name, gc=gc)
+# If expr is a stored temp array, then re-use its name
+  if(!is.null(ans@gc$temp) && ans@gc$temp && missing(name)) name=ans@name
+  .scidbeval(ans@name, `eval`=eval, name=name, gc=gc, schema=ans@schema, temp=temp)
 }
 
 # Create a new scidb reference to an existing SciDB array.
@@ -56,11 +59,16 @@ scidb = function(name, gc, `data.frame`)
   if(missing(gc)) gc=FALSE
   if(is.scidb(name) || is.scidbdf(name))
   {
+    query = name@name
     return(.scidbeval(name@name, eval=FALSE, gc=gc, `data.frame`=`data.frame`, depend=list(name)))
   }
-  query = sprintf("show('filter(%s,true)','afl')",gsub("'","\\\\'",name,perl=TRUE))
-  schema = gsub("^.*<","<",iquery(query,`return`=1)$schema, perl=TRUE)
+  escape = gsub("'","\\\\'",name,perl=TRUE)
+  query = sprintf("join(show('filter(%s,true)','afl'), explain_logical('filter(%s,true)','afl'))", escape,escape)
+  query = iquery(query, `return`=TRUE)
+  logical_plan = query$logical_plan
+  schema = gsub("^.*<","<",query$schema, perl=TRUE)
   obj = scidb_from_schemastring(schema, name, `data.frame`)
+  obj@logical_plan = logical_plan
   if(gc)
   {
     if(length(grep("\\(",name))==0)
@@ -79,6 +87,22 @@ scidb = function(name, gc, `data.frame`)
   obj
 }
 
+# TRUE if the SciDB object 'name' or character name of a SciDB array is
+# a temporary SciDB array, FALSE otherwise.
+is.temp = function(name)
+{
+  if(is.scidb(name) || is.scidbdf(name))
+  {
+    if(!is.null(name@gc$temp)) return(name@gc$temp)
+    name = name@name
+  }
+  query = sprintf("filter(list('arrays'),name='%s')", name)
+  ans = iquery(query,return=TRUE)
+  if(nrow(ans)<1) return(FALSE)
+  if(is.null(ans$temporary)) return(FALSE)
+  ans$temporary == "true"
+}
+
 # An important internal convenience function that returns a scidb object.  If
 # eval=TRUE, a new SciDB array is created the returned scidb object refers to
 # that.  Otherwise, the returned scidb object represents a SciDB array promise.
@@ -95,25 +119,50 @@ scidb = function(name, gc, `data.frame`)
 # data.frame: (optional, logical) If TRUE, return a data.frame object, false
 #             return a scidb object. Default is missing, in which case an
 #             automatic decision is made about the object return class.
+# schema, temp: (optional) used to create SciDB temp arrays
+#               (requires scidb >= 14.8)
 #
 # OUTPUT
 # A `scidb` or `scidbdf` array object.
 #
 # NOTE
-# SciDB versions up to 13.12 only support AFL here. As of SciDB version
-# 14.3, expression can be AFL or AQL.
-`.scidbeval` = function(expr,eval,name,gc=TRUE, depend, `data.frame`)
+# Only AFL supported.
+`.scidbeval` = function(expr,eval,name,gc=TRUE, depend, `data.frame`, schema, temp)
 {
   ans = c()
   if(missing(depend)) depend=c()
+  if(missing(schema)) schema=""
+  if(missing(temp)) temp=FALSE
   if(!is.list(depend)) depend=list(depend)
+# Address bug #45. Try to cheaply determine if expr refers to a named array
+# or an AFL expression. If it's a named array, then eval must be set TRUE.
+  if(!grepl("\\(", expr, perl=TRUE)) eval = TRUE
   if(`eval`)
   {
-    if(missing(name)) newarray = tmpnam()
+    if(missing(name) || is.null(name))
+    {
+      newarray = tmpnam()
+      if(temp)
+      {
+# SciDB temporary array syntax varies with SciDB version
+        TEMP = "'TEMP'"
+        if(compare_versions(options("scidb.version")[[1]],14.12)) TEMP="true"
+        query   = sprintf("create_array(%s, %s, %s)", newarray, schema, TEMP)
+        iquery(query, `return`=FALSE)
+      }
+    }
     else newarray = name
     query = sprintf("store(%s,%s)",expr,newarray)
-    scidbquery(query, interrupt=TRUE)
+    scidbquery(query, interrupt=TRUE, stream=0L)
     ans = scidb(newarray,gc=gc,`data.frame`=`data.frame`)
+    if(temp) ans@gc$temp = TRUE
+# This is a fix for a SciDB issue that can unexpectedly change schema
+# bounds. And another fix to allow unexpected dimname and attribute name
+# changes. Arrgh.
+    if(schema!="" && !compare_schema(ans, schema, ignore_attributes=TRUE, ignore_dimnames=TRUE))
+    {
+      ans = repart(ans, schema)
+    }
   } else
   {
     ans = scidb(expr,gc=gc,`data.frame`=`data.frame`)
@@ -130,44 +179,62 @@ scidb = function(name, gc, `data.frame`)
 # store the connection information and obtain a unique ID
 scidbconnect = function(host=options("scidb.default_shim_host")[[1]],
                         port=options("scidb.default_shim_port")[[1]],
-                        username, password)
+                        username, password,
+                        auth_type=c("pam","digest"), protocol=c("http","https"))
 {
   scidbdisconnect()
+  auth_type = match.arg(auth_type)
+  protocol = match.arg(protocol)
   assign("host",host, envir=.scidbenv)
   assign("port",port, envir=.scidbenv)
+  assign("protocol",protocol,envir=.scidbenv)
   if(missing(username)) username=c()
   if(missing(password)) password=c()
-# Check for login
+# Check for login using either pam or basic authentication
   if(!is.null(username))
   {
-    auth = GET(resource="login",list(username=username, password=password),header=FALSE)
-    if(nchar(auth)<1) stop("Authentication error")
-    assign("auth",auth,envir=.scidbenv)
+    if(auth_type=="pam")
+    {
+      auth = GET(resource="login",list(username=username, password=password),header=FALSE)
+      if(nchar(auth)<1) stop("Authentication error")
+      assign("auth",auth,envir=.scidbenv)
+    } else # HTTP basic digest auth
+    {
+      assign("digest",paste(username,password,sep=":"),envir=.scidbenv)
+    }
+    assign("authtype",auth_type,envir=.scidbenv)
     assign("authenv",new.env(),envir=.scidbenv)
     reg.finalizer(.scidbenv$authenv, function(e) scidblogout(), onexit=TRUE)
   }
 
 # Use the query ID from a query as a unique ID for automated
 # array name generation.
-  x = scidbquery(query="setopt('precision','16')",release=1,resp=TRUE)
+  x = tryCatch(
+        scidbquery(query="setopt('precision','16')",release=1,resp=TRUE,stream=0L),
+        error=function(e) stop("Connection error"))
   if(is.null(.scidbenv$uid))
   {
-    id = strsplit(x$response, split="\\r\\n")[[1]]
+    id = tryCatch(strsplit(x$response, split="\\r\\n")[[1]],
+           error=function(e) stop("Connection error"))
     id = id[[length(id)]]
     assign("uid",id,envir=.scidbenv)
   }
 # Try to load the dense_linear_algebra library
   tryCatch(
     scidbquery(query="load_library('dense_linear_algebra')",
-               release=1,resp=FALSE),
+               release=1,resp=FALSE, stream=0L),
     error=invisible)
 # Try to load the example_udos library (>= SciDB 13.6)
   tryCatch(
-    scidbquery(query="load_library('example_udos')",release=1,resp=FALSE),
+    scidbquery(query="load_library('example_udos')",release=1,resp=FALSE,stream=0L),
+    error=invisible)
+# Try to load the superfunpack
+  tryCatch(
+    scidbquery(query="load_library('superfunpack')",release=1,resp=FALSE,stream=0L),
     error=invisible)
 # Try to load the P4 library
   tryCatch(
-    scidbquery(query="load_library('linear_algebra')",release=1,resp=FALSE),
+    scidbquery(query="load_library('linear_algebra')",release=1,resp=FALSE,stream=0L),
     error=invisible)
 # Save available operators
   assign("ops",iquery("list('operators')",return=TRUE),envir=.scidbenv)
@@ -195,11 +262,10 @@ scidblogout = function()
 
 scidbdisconnect = function()
 {
+# forces call to scidblogout via finalizer, see scidbconnect:
   if(exists("authenv",envir=.scidbenv)) rm("authenv",envir=.scidbenv)
   gc()
-  if(exists("auth",envir=.scidbenv))  rm("auth",envir=.scidbenv)
-  if(exists("host",envir=.scidbenv)) rm("host", envir=.scidbenv)
-  if(exists("port",envir=.scidbenv)) rm("port", envir=.scidbenv)
+  rm(list=ls(envir=.scidbenv), envir=.scidbenv)
 }
 
 make.names_ = function(x)
@@ -235,6 +301,15 @@ getSession = function()
   session
 }
 
+# Returns the eponymous package option setting used to globally overried
+# interrupt handling. TRUE means abide by interrupts, FALSE ignore.
+scidb.interrupt = function()
+{
+  ans = options("scidb.interrupt")[[1]]
+  if(is.null(ans)) ans=TRUE
+  tryCatch(as.logical(ans), error=function(e) TRUE)
+}
+
 # Supply the base SciDB URI from the global host, port and auth
 # parameters stored in the .scidbenv package environment.
 # Every function that needs to talk to the shim interface should use
@@ -247,7 +322,7 @@ URI = function(resource="", args=list())
   if(!exists("host",envir=.scidbenv)) stop("Not connected...try scidbconnect")
   if(exists("auth",envir=.scidbenv))
     args = c(args,list(auth=get("auth",envir=.scidbenv)))
-  prot = "http://"
+  prot = paste(get("protocol",envir=.scidbenv),"//",sep=":")
   if("username" %in% names(args) || "auth" %in% names(args)) prot = "https://"
   ans  = paste(prot, get("host",envir=.scidbenv),":",get("port",envir=.scidbenv),sep="")
   ans = paste(ans, resource, sep="/")
@@ -260,36 +335,46 @@ URI = function(resource="", args=list())
 # resource: URI service name
 # args: named list of HTTP GET query parameters
 # header: TRUE=return the HTTP response header, FALSE=don't return it
-# async: Doesn't really do anything
+# async: Doesn't really do anything, should be removed XXX
 # interrupt: Set to FALSE to disable SIGINT, otherwise handle gracefully.
-GET = function(resource, args=list(), header=TRUE, async=FALSE, interrupt=FALSE)
+# ...: additional curl options passed to getURI
+#
+# returns NULL if async=TRUE or the return value of RCurl getURI otherwise
+GET = function(resource, args=list(), header=TRUE, async=FALSE, interrupt=FALSE, ...)
 {
   if(!(substr(resource,1,1)=="/")) resource = paste("/",resource,sep="")
   uri = URI(resource, args)
-  uri = URLencode(uri)
+  uri = oldURLencode(uri)
   uri = gsub("\\+","%2B",uri,perl=TRUE)
   on.exit(sigint(SIG_DFL)) # Reset signal handler on exit
   callback = curl_signal_trap
+  interrupt = interrupt && scidb.interrupt()   # Check package override
   if(interrupt)
   {
     sigreset ()
-    sigint(SIG_TRP)          # Handle SIGINT
+    sigint(TRAP())          # Handle SIGINT
   } else
   {
     sigint(SIG_IGN)          # Ignore SIGINT
   }
+#  digest = curlopts()
+  digest = digest_auth("GET",uri)
   if(async)
   {
     getURI(url=uri,
-      .opts=list(header=header,
+      .opts=c(list(header=header,
                  'ssl.verifyhost'=as.integer(options("scidb.verifyhost")),
                  'ssl.verifypeer'=0,
                  noprogress=!interrupt, progressfunction=curl_signal_trap),
+                 list(...), httpheader=digest),
       async=TRUE)
     return(NULL)
   }
-  getURI(url=uri, .opts=list(header=header,'ssl.verifyhost'=as.integer(options("scidb.verifyhost")),'ssl.verifypeer'=0,
-                         noprogress=!interrupt, progressfunction=curl_signal_trap))
+# The gsub is here because basic digest authentication might return TWO
+# concatenated responses: the first a 401 auth error as the digest
+# authentication is negotiated and the 2nd potentially the desired response (or
+# maybe another error). We strip everything up to the last response with gsub.
+  gsub(".*\\r\\n\\r\\nHTTP","HTTP",getURI(url=uri, .opts=c(list(header=header,'ssl.verifyhost'=as.integer(options("scidb.verifyhost")),'ssl.verifypeer'=0, noprogress=!interrupt, progressfunction=curl_signal_trap),list(...),httpheader=digest)))
 }
 
 # Check if array exists
@@ -320,7 +405,6 @@ type= c("arrays","operators","functions","types","aggregates","instances","queri
     if(!missing(pattern))
       z = grep(z,pattern=pattern,value=TRUE)
   }
-  if(type=="arrays" && verbose) z=z[,3]
   z
 }
 scidbls = function(...) scidblist(...)
@@ -333,6 +417,8 @@ scidbls = function(...) scidblist(...)
 # release: Set to zero preserve web session until manually calling release_session
 # session: if you already have a SciDB http session, set this to it, otherwise NULL
 # resp(logical): return http response
+# interrupt: Set to TRUE to enable user interrupt
+# stream: Set to 0L or 1L to control streaming, otherwise use package options
 # Example values of save:
 # save="dcsv"
 # save="lcsv+"
@@ -340,10 +426,15 @@ scidbls = function(...) scidblist(...)
 #
 # Returns the HTTP session in each case
 scidbquery = function(query, afl=TRUE, async=FALSE, save=NULL,
-                      release=1, session=NULL, resp=FALSE, interrupt=FALSE)
+                      release=1, session=NULL, resp=FALSE, interrupt=FALSE, stream)
 {
   DEBUG = FALSE
+  STREAM = 0L
   if(!is.null(options("scidb.debug")[[1]]) && TRUE==options("scidb.debug")[[1]]) DEBUG=TRUE
+  if(missing(stream))
+  {
+    if(!is.null(options("scidb.stream")[[1]]) && TRUE==options("scidb.stream")[[1]]) STREAM=1L
+  } else STREAM = as.integer(stream)
   sessionid=session
   if(is.null(session))
   {
@@ -360,11 +451,11 @@ scidbquery = function(query, afl=TRUE, async=FALSE, save=NULL,
     {
       if(is.null(save))
         GET("/execute_query",list(id=sessionid,release=release,
-             query=query,afl=as.integer(afl)),
+             query=query,afl=as.integer(afl), stream=0L),
              interrupt=interrupt)
       else
         GET("/execute_query",list(id=sessionid,release=release,
-            save=save,query=query,afl=as.integer(afl)), 
+            save=save,query=query,afl=as.integer(afl),stream=STREAM), 
             interrupt=interrupt)
     }, error=function(e)
     {
@@ -389,7 +480,7 @@ scidbquery = function(query, afl=TRUE, async=FALSE, save=NULL,
   {
     stop(strsplit(w,"\r\n\r\n")[[1]][[2]])
   }
-  if(DEBUG) print(proc.time()-t1)
+  if(DEBUG) cat("Query time",(proc.time()-t1)[3],"\n")
   if(resp) return(list(session=sessionid, response=ans))
   sessionid
 }
@@ -409,6 +500,7 @@ scidbremove.default = function(x, error=warning, async, force, warn=TRUE, recurs
   if(is.null(x)) return(invisible())
   if(missing(async)) async=FALSE
   if(missing(force)) force=FALSE
+  errfun=error
 
   safe = options("scidb.safe_remove")[[1]]
   if(is.null(safe)) safe = TRUE
@@ -427,12 +519,12 @@ scidbremove.default = function(x, error=warning, async, force, warn=TRUE, recurs
     query = sprintf("remove(%s)",y)
     if(grepl(sprintf("^R_array.*%s$",uid),y,perl=TRUE))
     {
-      tryCatch( scidbquery(query,async=async, release=1),
-                error=function(e) if(!recursive && warn)print(e))
+      tryCatch( scidbquery(query,async=async, release=1, stream=0L),
+                error=function(e) if(!recursive && warn)errfun(e))
     } else if(force)
     {
-      tryCatch( scidbquery(query,async=async, release=1),
-                error=function(e) if(!recursive && warn)print(e))
+      tryCatch( scidbquery(query,async=async, release=1, stream=0L),
+                error=function(e) if(!recursive && warn)errfun(e))
     } else if(warn)
     {
       warning("The array ",y," is protected from easy removal. Specify force=TRUE if you really want to remove it.")
@@ -496,7 +588,17 @@ df2scidb = function(X,
       else if("character" %in% class(X[,j])) 
         typ[j] = paste("string",nullable[j])
       else if("factor" %in% class(X[,j])) 
-        typ[j] = paste("string",nullable[j])
+      {
+        if("scidb_factor" %in% class(X[,j]))
+        {
+          typ[j] = paste("int64 NULL")
+          Xj = X[,j]
+          levels(Xj) = levels_scidb(Xj)
+          X[,j] = as.vector(Xj)
+        }
+        else
+          typ[j] = paste("string",nullable[j])
+      }
       else if("POSIXct" %in% class(X[,j])) 
       {
         warning("Converting R POSIXct to SciDB datetime as UTC time. Subsecond times rounded to seconds.")
@@ -525,14 +627,15 @@ df2scidb = function(X,
 
 # Post the input string to the SciDB http service
   uri = URI("upload_file",list(id=session))
-  tmp = postForm(uri=uri, uploadedfile=fileUpload(contents=scidbInput,filename="scidb",contentType="application/octet-stream"),.opts=curlOptions(httpheader=c(Expect=""),'ssl.verifyhost'=as.integer(options("scidb.verifyhost")),'ssl.verifypeer'=0))
+  hdr = digest_auth("POST",uri)
+  tmp = tryCatch(postForm(uri=uri, uploadedfile=fileUpload(contents=scidbInput,filename="scidb",contentType="application/octet-stream"),.opts=curlOptions(httpheader=hdr,'ssl.verifyhost'=as.integer(options("scidb.verifyhost")),'ssl.verifypeer'=0)), error=invisible)
   tmp = tmp[[1]]
   tmp = gsub("\r","",tmp)
   tmp = gsub("\n","",tmp)
 
 # Load query
   query = sprintf("store(input(%s, '%s'),%s)",SCHEMA, tmp, name)
-  scidbquery(query, async=FALSE, release=1, session=session)
+  scidbquery(query, async=FALSE, release=1, session=session, stream=0L)
   scidb(name,`data.frame`=TRUE,gc=gc)
 }
 
@@ -575,8 +678,8 @@ df2scidb = function(X,
   fn = tempfile()
   bytes = writeBin(.Call("scidb_raw",as.vector(t(matrix(c(X@i + start[[1]],j + start[[2]], X@x),length(X@x)))),PACKAGE="scidb"),con=fn)
   url = URI("/upload_file",list(id=session))
-  ans = postForm(uri = url, uploadedfile = fileUpload(filename=fn),
-                 .opts = curlOptions(httpheader = c(Expect = ""),'ssl.verifyhost'=as.integer(options("scidb.verifyhost")),'ssl.verifypeer'=0))
+  hdr = digest_auth("POST",url)
+  ans = tryCatch(postForm(uri = url, uploadedfile = fileUpload(filename=fn), .opts=curlOptions(httpheader=hdr,'ssl.verifyhost'=as.integer(options("scidb.verifyhost")),'ssl.verifypeer'=0)), error=invisible)
   unlink(fn)
   ans = ans[[1]]
   ans = gsub("\r","",ans)
@@ -590,8 +693,10 @@ df2scidb = function(X,
 
 iquery = function(query, `return`=FALSE,
                   afl=TRUE, iterative=FALSE,
-                  n=10000, excludecol, ...)
+                  n=10000, excludecol, binary=FALSE, ...)
 {
+  DEBUG = FALSE
+  if(!is.null(options("scidb.debug")[[1]]) && TRUE==options("scidb.debug")[[1]]) DEBUG=TRUE
   if(!afl && `return`) stop("return=TRUE may only be used with AFL statements")
   if(iterative && !`return`) stop("Iterative result requires return=TRUE")
   if(is.scidb(query) || is.scidbdf(query))  query=query@name
@@ -606,24 +711,30 @@ iquery = function(query, `return`=FALSE,
   if(n==Inf) n = -1    # Indicate to shim that we want all the lines of output
   for(query in qsplit)
   {
+# Only return the last query, mimicking command-line iquery.
     if(`return` && m==length(qsplit))
     {
+      if(binary)
+      {
+        return(scidb_unpack_to_dataframe(query,...))
+      }
       ans = tryCatch(
        {
         sessionid = scidbquery(query,afl,async=FALSE,save="lcsv+",release=0, interrupt=TRUE)
+        dt1 = proc.time()
         result = tryCatch(
           {
             GET("/read_lines",list(id=sessionid,n=as.integer(n+1)),header=FALSE,interrupt=TRUE)
           },
           error=function(e)
           {
-# Not used yet. It will be important when shim is modified to stream
-# results back. XXX TODO
-#            GET("/cancel",list(id=sessionid))
+             GET("/cancel",list(id=sessionid))
              GET("/release_session",list(id=sessionid))
              stop(e)
           })
         GET("/release_session",list(id=sessionid))
+        if(DEBUG) cat("Data transfer time",(proc.time()-dt1)[3],"\n")
+        dt1 = proc.time()
 # Handle escaped quotes
         result = gsub("\\\\'","''",result, perl=TRUE)
         result = gsub("\\\\\"","''",result, perl=TRUE)
@@ -637,12 +748,16 @@ iquery = function(query, `return`=FALSE,
                 read.table(val,sep=",",stringsAsFactors=FALSE,header=TRUE,...)},
                 error=function(e){ warning(e);c()})
         close(val)
+        if(DEBUG) cat("R parsing time",(proc.time()-dt1)[3],"\n")
         ret
        }, error = function(e)
            {
              stop(e)
            })
-    } else ans = scidbquery(query,afl)
+    } else
+    {
+      ans = scidbquery(query, afl, async=FALSE, release=1, interrupt=TRUE, stream=0L)
+    }
     m = m + 1
   }
   if(!(`return`)) return(invisible())
@@ -799,8 +914,22 @@ origin = function(x)
   .scidbeval(query,`eval`=FALSE,depend=list(x),gc=TRUE)
 }
 
-chunk_map = function()
+
+chunk_map = function(`array`)
 {
+  if(!missing(`array`))
+  {
+    if(is.scidb(`array`) || is.scidbdf(`array`))
+    {
+      `array` = schema(`array`)
+    }
+    q1="redimension( apply(list('chunk map'), element_count, int64(nelem), iid, int64(inst), aid, int64(arrid)), <element_count:int64, iid:int64>[aid=0:*,1000,0,i=0:*,1000000000,0])"
+    q2=sprintf("redimension( filter(apply( list('arrays', true), aid, int64(id), match, regex(name,'%s@*[0-9]*')), match=true), <name: string null> [aid = 0:*,1000,0])", `array`)
+    query=sprintf("project( cross_join(%s as X, %s as Y, X.aid, Y.aid), element_count, iid)", q1, q2)
+    return(iquery(query,`return`=TRUE)[,2:4])
+  }
+
+
 iquery("
 project(
  cross_join(
@@ -860,7 +989,7 @@ scidbdfcc = function(x)
 #
 # Here is an example:
 #
-#    sigint(SIG_TRP)
+#    sigint(TRAP())
 #    x=getBinaryURL(url,
 #        .opts=list(noprogress=FALSE, progressfunction=curl_signal_trap))
 #    sigint(SIG_DFL)
@@ -882,7 +1011,7 @@ sigreset = function()
 curl_signal_trap = function(down,up)
 {
 # Unforunately, RStudio doesn't seem to let us set up custom signal handlers.
-  if(SIG_TRP == SIG_IGN)
+  if(TRAP() == SIG_IGN)
   {
     k = tryCatch(
     {
@@ -946,3 +1075,216 @@ show_commit_log = function()
     file.show(log)
   }
 }
+
+# This is a workhorse SciDB data munger
+# query is a query string, not a scidb object.
+scidb_unpack_to_dataframe = function(query, ...)
+{
+  DEBUG = FALSE
+  if(!is.null(options("scidb.debug")[[1]]) && TRUE==options("scidb.debug")[[1]]) DEBUG=TRUE
+  buffer = 100000L
+  row_names = NULL
+  args = list(...)
+  interrupt = scidb.interrupt()
+  if(!is.null(args$buffer))
+  {
+    argsbuf = tryCatch(as.integer(args$buffer),warning=function(e) NA)
+# Potential problem here if args$buffer is bogus or '*' or whatever.
+# Deal with it. XXX FIX ME
+    if(!is.na(argsbuf) && argsbuf < 100e6) buffer = as.integer(argsbuf)
+  }
+  if(!is.null(args$row.names)) row_names = args$row.names
+  up = TRUE
+  if(!is.null(args$unpack)) up = args$unpack
+  if(up) x = scidb(sprintf("unpack(%s,row)",query), data.frame=TRUE)
+  else x = scidb(sprintf("project(unpack(%s,row),%s)",query,scidb_attributes(x)), data.frame=TRUE)
+  N = scidb_nullable(x)
+  TYPES = scidb_types(x)
+  ns = rep("",length(N))
+  ns[N] = "null"
+  format_string = paste(paste(TYPES,ns),collapse=",")
+  format_string = sprintf("(%s)",format_string)
+  sessionid = scidbquery(x@name, save=format_string, async=FALSE, release=0, interrupt=TRUE)
+# Release the session on exit
+  on.exit( GET("/release_session",list(id=sessionid)) ,add=TRUE)
+
+  dt2 = proc.time()
+  r = URI("/read_bytes",list(id=sessionid,n=0))
+  sigint(TRAP())
+  BUF = tryCatch(
+        { 
+          getBinaryURL(r, .opts=c(curlopts(),list('ssl.verifyhost'=as.integer(options("scidb.verifyhost")),'ssl.verifypeer'=0, noprogress=!interrupt, progressfunction=curl_signal_trap)))
+        }, error=function(e)
+        { 
+          GET("/release_session",list(id=sessionid))
+          sigint(SIG_DFL)
+          stop("canceled")
+        })
+  sigint(SIG_DFL)
+  if(DEBUG) cat("Data transfer time",(proc.time()-dt2)[3],"\n")
+  dt1 = proc.time()
+  len = length(BUF)
+  p = 0
+  ans = c()
+  cnames = c(names(x),"lines","p")
+  rnames = c()
+  while(p<len)
+  {
+dt2 = proc.time()
+    n = ncol(x)
+    tmp   = .Call("scidb_parse", as.integer(buffer), TYPES, N, BUF, as.double(p))
+    names(tmp) = cnames
+    lines = tmp[[n+1]]
+    p_old = p
+    p     = tmp[[n+2]]
+if(DEBUG) cat("  R buffer ",p,"/",len," bytes parsing time",(proc.time()-dt2)[3],"\n")
+dt2 = proc.time()
+    if(lines>0)
+    {
+      len_out = length(tmp[[1]])
+      if(!is.null(row_names))
+      {
+        if(is.numeric(row_names) && length(row_names)==1)
+        {
+          rnames = c(rnames,tmp[[row_names]][1:lines])
+          tmp = tmp[-row_names]
+          n = n -1
+        }
+      }
+      if(lines < len_out) tmp = lapply(tmp[1:n],function(x) x[1:lines])
+# Let's adaptively re-estimate a buffer size
+      avg_bytes_per_line = ceiling((p - p_old)/lines)
+      buffer = ceiling(1.3*(len - p)/avg_bytes_per_line) # Engineering factor
+# Assemble the data frame
+      if(is.null(ans)) ans = data.frame(tmp[1:n],stringsAsFactors=FALSE)
+      else ans = rbind(ans,data.frame(tmp[1:n],stringsAsFactors=FALSE))
+    }
+if(DEBUG) cat("  R rbind/df assembly time",(proc.time()-dt2)[3],"\n")
+  }
+dt2 = proc.time()
+  if(!is.null(row_names))
+  {
+    if(is.numeric(row_names) && length(row_names)==1)
+    {
+      rownames(ans) = rnames
+    } else
+    {
+      rownames(ans) = row_names
+    }
+  }
+if(DEBUG) cat("  R rownames assignment",(proc.time()-dt2)[3],"\n")
+  if(DEBUG) cat("Total R parsing time",(proc.time()-dt1)[3],"\n")
+  ans
+}
+
+# Convenience function for digest authentication.
+digest_auth = function(method, uri, realm="", nonce="123456")
+{
+  if(exists("authtype",envir=.scidbenv))
+  {
+   if(.scidbenv$authtype != "digest") return(c(Expect=""))
+  }
+  uri = gsub(".*/","/",uri)
+  userpwd = .scidbenv$digest
+  if(is.null(userpwd)) userpwd=":"
+  up = strsplit(userpwd,":")[[1]]
+  user = up[1]
+  pwd  = up[2]
+  if(is.na(pwd)) pwd=""
+  ha1=digest(sprintf("%s:%s:%s",user,realm,pwd,algo="md5"),serialize=FALSE)
+  ha2=digest(sprintf("%s:%s", method,  uri, algo="md5"),serialize=FALSE)
+  cnonce="MDc1YmFhOWFkY2M0YWY2MDAwMDBlY2JhMDAwMmYxNTI="
+  nc="00000001"
+  qop="auth"
+  response=digest(sprintf("%s:%s:%s:%s:%s:%s",ha1,nonce,nc,cnonce,qop,ha2),algo="md5",serialize=FALSE)
+  sprintf('Authorization: Digest username="%s", realm=%s, nonce="%s", uri="%s", cnonce="%s", nc=%s, qop=%s, response="%s"', user, realm, nonce, uri, cnonce, nc, qop, response)
+}
+
+# Return extra curl options defined by the active connection
+curlopts = function()
+{
+  ans = c()
+  if(exists("digest",envir=.scidbenv))
+    ans=c(ans,list(userpwd=get("digest",envir=.scidbenv)))
+  ans
+}
+
+# Internal warning function
+warnonce = (function() {
+  state = list(
+    unpack="The array contains multiple SciDB attributes, returning as an unpacked dataframe.",
+    missing="An array might contain missing values (R NA/SciDB 'null'-able attribute).\n Missing values are not yet understood by SciDB multiplication operators\n and any missing values have been replaced with zero (see replaceNA).",
+    nonum="Note: The R sparse Matrix package does not support certain value types like\ncharacter strings",
+    toobig="Dimensions are too big! Returning data in unpacked data.frame form."
+  )
+  function(warn) {
+    if(!is.null(state[warn][[1]])) {
+      warning(state[warn], call.=FALSE)
+      s <<- state
+      s[warn] = c()
+      state <<- s
+    }
+  }
+}) () # oh boy
+
+
+# Utility function that parses an aggregation expression
+# makes sense of statements like
+# max(a) as amax, avg(b) as bmean, ...
+# Input
+# x: scidb object
+# expr: aggregation expression (as is required)
+# Output
+# New attribute schema
+aparser = function(x, expr)
+{
+  map = list(
+    ApproxDC    = function(x) "double null",
+    avg         = function(x) "double null",
+    count       = function(x) "uint64 null",
+    first_value = function(x) sprintf("%s null",x),
+    last_value  = function(x) sprintf("%s null",x),
+    mad         = function(x) sprintf("%s null",x),
+    max         = function(x) sprintf("%s null",x),
+    min         = function(x) sprintf("%s null",x),
+    median      = function(x) sprintf("%s null",x),
+    prod        = function(x) sprintf("%s null",x),
+    stdev       = function(x) "double null",
+    sum         = function(x) sprintf("%s null",x),
+    top_five    = function(x) sprintf("%s null",x),
+    var         = function(x) "double null")
+
+  p = strsplit(expr,",")[[1]]                      # comma separated
+  p = lapply(p,function(v) strsplit(v,"as")[[1]])  # as is required
+  new_attrs = lapply(p, function(v) v[[2]])        # output attribute names
+  y = lapply(p,function(v)strsplit(v,"\\(")[[1]])  #
+  fun = lapply(y, function(v) gsub(" ","",v[[1]])) # functions
+  old_attrs = unlist(lapply(y,function(v)gsub("[\\) ]","",v[[2]])))
+  i = 1:length(x@attributes)
+  names(i)=x@attributes
+  old_types = scidb_types(x)[i[old_attrs]]
+  old_types[is.na(old_types)] = "int64"    # catch all
+  z = rep("",length(new_attrs))
+  for(j in 1:length(z))
+  {
+    z[j] = sprintf("%s:%s", new_attrs[j],map[unlist(fun)][[j]](old_types[j]))
+  }
+  sprintf("<%s>", paste(z, collapse=","))
+}
+
+# Some versions of RCurl seem to contain a broken URLencode function.
+oldURLencode = function (URL, reserved = FALSE) 
+{
+    OK <- paste0("[^-ABCDEFGHIJKLMNOPQRSTUVWXYZ", "abcdefghijklmnopqrstuvwxyz0123456789$_.+!*'(),", 
+        if (!reserved) 
+            ";/?:@=&", "]")
+    x <- strsplit(URL, "")[[1L]]
+    z <- grep(OK, x)
+    if (length(z)) {
+        y <- sapply(x[z], function(x) paste0("%", as.character(charToRaw(x)), 
+            collapse = ""))
+        x[z] <- y
+    }
+    paste(x, collapse = "")
+}
+
